// AN_190_C232HM_libMPSSE_I2C.cpp : Defines the entry point for the console application.
//
/*
========================================================================
Copy the following files from the libMPSSE folder to your project folder
- ftd2xx.h		// This is the base FTDI USB header
- libMPSSE.h	// This is the header to define the libMPSSE calls
- libMPSSE.dll	// This is the DLL
- libMPSSE.a	// This is the libarary (as opposed to a .lib file)
- ftd2xx.lib	// This is the main D2XX library - only needed to set
				// the "only drive zero" feature of the FT232H

Next, open the Project menu item and select <Project Name> Properties
Under the Linker tree, select Input.  
Add libmpsse.a and ftd2xx.lib to the Additional Dependencies
========================================================================
*/
// Included by Visual Studio for a standard console app
#include "stdafx.h"

// Include standard windows and C++ headers
#include <windows.h>
#include <stdio.h>
#include <conio.h>

// Include the header files for the base FTDI and libMPSSE libraries
#include "ftd2xx.h"
#include "libMPSSE_i2c.h"

int _tmain(int argc, _TCHAR* argv[])
{
	// General variables
	uint32 i, index;
	unsigned char j;

	// MAX17061 variable for SMBus address
	uint32 i2cAddr;

	// libMPSSE-related variables
	FT_STATUS i2cStatus = 0;
	uint32 i2cNumChannels = 0;
	FT_DEVICE_LIST_INFO_NODE i2cChannelInfo[4];
	FT_HANDLE i2cHandle;
	ChannelConfig i2cChannel;

	// This is the I2C address (fixed at 0101100 for the MAX17061 
	// Note this is right-justified for the libMPSSE calls.  Bit 7 is unused.)
	i2cAddr = 0x2C;						// This is the default address

	unsigned char i2cData[16];			// Initialize a data buffer
	uint32 i2cNumBytesToTx;				// Initialize a counter for the buffer
	uint32 i2cNumBytesTxd;				// Initialize another counter for the number of bytes actually sent
	uint32 i2cNumBytesToRx;
	uint32 i2cNumBytesRxd;

	// Initialize the MPSSE libraries
	Init_libMPSSE();

	// First, find out how many devices are capable of MPSSE commands
	i2cStatus = I2C_GetNumChannels(&i2cNumChannels);
	if (i2cStatus)						// Any value besides zero is an error
	{
		printf("ERROR %X - Cannot execute I2C_GenNumChannels.  \nPress <Enter> to continue.\n", i2cStatus);
		getchar();
		return 1;
	}

	if (i2cNumChannels > 4)
	{
		printf("This program can only identify up to 4 MPSSE devices\n");
		getchar();
		return 1;
	}

	if (i2cNumChannels < 1)
	{
		printf("No C232HM cables were found\n");
		getchar();
		return 1;
	}

	// Next, get the information about each one
	for (i=0; i < i2cNumChannels; i++)
	{
		// Get the information from each MPSSE device
		i2cStatus = I2C_GetChannelInfo(i, &i2cChannelInfo[i]);
		if (i2cStatus)
		{
			printf("ERROR %X - Cannot execute I2C_GetChannelInfo.  \nPress <Enter> to continue.\n", i2cStatus);
			getchar();
			return 1;
		}

		// If it's one of the C232HM calbes, then stop - we'll take the first one we find
		if ((i2cChannelInfo[i].Description == "CH232HM-EDHSL-0") | (i2cChannelInfo[i].Description == "CH232HM-DDHSL-0"))
			break;
	}

	index = i - 1;	// The index is zero-based
	
	// Nofity what was found and what we're using
	printf("\n");
	printf("  %d MPSSE devices found.  Device %d will be used. \n", i2cNumChannels, index);
	printf("\n");
	
	// We found the cable, now let's open it

	i2cStatus = I2C_OpenChannel(index, &i2cHandle);
	if (i2cStatus) 
	{
		printf("ERROR %X - Cannot execute I2C_OpenChannel.  \nPress <Enter> to continue.\n", i2cStatus);
		getchar();
		return 1;
	}

	// Set up the various I2C parameters
	i2cChannel.ClockRate = I2C_CLOCK_STANDARD_MODE;		// 100Kbps
	i2cChannel.LatencyTimer = 1;						// 1mS latency timer
	i2cChannel.Options = 0;								// 3-phase clocking enabled

	i2cStatus = I2C_InitChannel(i2cHandle, &i2cChannel);
	if (i2cStatus) 
	{
		printf("ERROR %X - Cannot execute I2C_InitChannel.  \nPress <Enter> to continue.\n", i2cStatus);
		getchar();
		return 1;
	}

	// For the FT232H, the "write on zero only" command is not yet implemented.
	// These commands will enable this feature
	i2cNumBytesToTx = 0;	// Initialize data counter
	i2cData[i2cNumBytesToTx++] = 0x9E;		// Enable write zero only
	i2cData[i2cNumBytesToTx++] = 0x03;		// ADBUS0 and ADBUS1 only
	i2cData[i2cNumBytesToTx++] = 0x00;

	unsigned long sent;
	FT_Write(i2cHandle, i2cData, i2cNumBytesToTx, &sent);

	// Now get the read-only registers of the MAX17061 device

	// The MAX17061 uses SMBus Read Byte and Write Byte commands.  This requires a combination of
	// two i2cMPSSE commands.  The first is a write to send the address and "command" (or register address).
	// The second is the actual read.  Note that the first has no stop condition, but the second has another
	// start condition
	i2cNumBytesToTx = 0;					// Initialize data counter
	i2cData[i2cNumBytesToTx++] = 0x03;		// ID register
	i2cStatus = I2C_DeviceWrite(i2cHandle, i2cAddr, i2cNumBytesToTx, i2cData, &i2cNumBytesTxd, 0x01);
	i2cNumBytesToRx = 1;
	i2cStatus += I2C_DeviceRead(i2cHandle, i2cAddr, i2cNumBytesToRx, &i2cData[0], &i2cNumBytesRxd, 0x03);
	if (i2cStatus) 
	{
		printf("ERROR %X - Cannot read MAX17061 ID register.  \nPress <Enter> to continue.\n", i2cStatus);
		getchar();
		return 1;
	}

	printf("  The ID register value is 0x%X\n", i2cData[0]);
	printf("\n");

	i2cNumBytesToTx = 0;					// Initialize data counter
	i2cData[i2cNumBytesToTx++] = 0x02;		// Status register
	i2cStatus = I2C_DeviceWrite(i2cHandle, i2cAddr, i2cNumBytesToTx, i2cData, &i2cNumBytesTxd, 0x01);
	i2cNumBytesToRx = 1;
	i2cStatus += I2C_DeviceRead(i2cHandle, i2cAddr, i2cNumBytesToRx, &i2cData[0], &i2cNumBytesRxd, 0x03);
	if (i2cStatus) 
	{
		printf("ERROR %X - Cannot read MAX17061 Status register.  \nPress <Enter> to continue.\n", i2cStatus);
		getchar();
		return 1;
	}

	printf("  The Status register value is 0x%X\n", i2cData[0]);
	printf("\n");

	// Now let's set up the brightness to the lowest level so we're not blinded

	// As noted, the MAX17061 uses SMBus Read Byte and Write Byte.  Here only the Write Byte, which is 
	// accomplished by a single libMPSSE write

	i2cNumBytesToTx = 0;	// Initialize data counter
	i2cData[i2cNumBytesToTx++] = 0x00;		// brightness
	i2cData[i2cNumBytesToTx++] = 0x00;		// lowest level, about 2.5% according to the MAX17061 datasheet
	i2cStatus = I2C_DeviceWrite(i2cHandle, i2cAddr, i2cNumBytesToTx, i2cData, &i2cNumBytesTxd, 0x03);
	if (i2cStatus) 
	{
		printf("ERROR %X - Cannot set MAX17061 brightness register.  \nPress <Enter> to continue.\n", i2cStatus);
		getchar();
		return 1;
	}

	// Finally, we'll turn on the LEDs

	i2cNumBytesToTx = 0;	// Initialize data counter
	i2cData[i2cNumBytesToTx++] = 0x01;		// Control Reg
	i2cData[i2cNumBytesToTx++] = 0x01;		// Turn on LEDs
	i2cStatus = I2C_DeviceWrite(i2cHandle, i2cAddr, i2cNumBytesToTx, i2cData, &i2cNumBytesTxd, 0x03);
	if (i2cStatus) 
	{
		printf("ERROR %H - Cannot set MAX17061 control register.  \nPress <Enter> to continue.\n", i2cStatus);
		getchar();
		return 1;
	}

	// The brightness can be adjusted on-the-fly, so lets go from dim to bright to dim

	printf("  Ramping up LED brightness...\n");
	printf("\n");
	for (j = 0; j < 240 ; j = j + 10)				
	{
		i2cNumBytesToTx = 0;	// Initialize data counter
		i2cData[i2cNumBytesToTx++] = 0x00;		// brightness
		i2cData[i2cNumBytesToTx++] = j + 10;
		i2cStatus = I2C_DeviceWrite(i2cHandle, i2cAddr, i2cNumBytesToTx, i2cData, &i2cNumBytesTxd, 0x03);
		if (i2cStatus) 
		{
			printf("Loop Number %X, ERROR %X - Cannot set MAX17061 brightness register in ramp-up.  \nPress <Enter> to continue.\n", j, i2cStatus);
			getchar();
			return 1;
		}
		Sleep(100);
	}

	printf("  Ramping down LED brightness...\n");
	printf("\n");
	for (j = 250; j > 0; j = j - 10)
	{
		i2cNumBytesToTx = 0;	// Initialize data counter
		i2cData[i2cNumBytesToTx++] = 0x00;		// brightness
		i2cData[i2cNumBytesToTx++] = j;
		i2cStatus = I2C_DeviceWrite(i2cHandle, i2cAddr, i2cNumBytesToTx, i2cData, &i2cNumBytesTxd, 0x03);
		if (i2cStatus) 
		{
			printf("Loop Number %X, ERROR %X - Cannot set MAX17061 brightness register in ramp-down.  \nPress <Enter> to continue.\n", j, i2cStatus);
			getchar();
			return 1;
		}
		Sleep(100);
	}

	// Turn off the LEDs
	printf("  Turning off the LEDs\n");
	printf("\n");
	i2cNumBytesToTx = 0;	// Initialize data counter
	i2cData[i2cNumBytesToTx++] = 0x01;		// Control Reg
	i2cData[i2cNumBytesToTx++] = 0x00;		// Turn on LEDs

	i2cStatus = I2C_DeviceWrite(i2cHandle, i2cAddr, i2cNumBytesToTx, i2cData, &i2cNumBytesTxd, 0x03);
	if (i2cStatus) 
	{
		printf("ERROR %X - Cannot set MAX17061 control register.  \nPress <Enter> to continue.\n", i2cStatus);
		getchar();
		return 1;
	}

	// Time to close up shop - close the channel, then ...
	printf("  Closing the CH232M MPSSE channel\n");
	printf("\n");
	i2cStatus = I2C_CloseChannel(i2cHandle);
	if (i2cStatus) 
	{
		printf("ERROR %X - Cannot execute I2C_CloseChannel.  \nPress <Enter> to continue.\n", i2cStatus);
		getchar();
		return 1;
	}

	// ... close out the MPSSE libraries
	printf("  Cleaning up the libMPSSE library\n");
	printf("\n");
	Cleanup_libMPSSE();
	
	printf("  Exit program.  \nPress <Enter> to continue.\n");
	printf("\n");
	getchar();
	return 0;
}

